//! `SeaORM` Entity. Generated by sea-orm-codegen 0.11.2

use rand::{rngs::OsRng, RngCore};
use scrypt::{scrypt, Params};
use sea_orm::{entity::prelude::*, ActiveValue::Set};

#[derive(Clone, Debug, PartialEq, DeriveEntityModel, Eq)]
#[sea_orm(table_name = "master_pass")]
pub struct Model {
    #[sea_orm(primary_key, auto_increment = false)]
    pub user_id: u64,
    #[sea_orm(column_type = "Binary(BlobSize::Blob(Some(64)))")]
    pub salt: Vec<u8>,
    #[sea_orm(column_type = "Binary(BlobSize::Blob(Some(64)))")]
    pub password_hash: Vec<u8>,
}

#[derive(Copy, Clone, Debug, EnumIter, DeriveRelation)]
pub enum Relation {}

impl ActiveModelBehavior for ActiveModel {}

fn hash_password(password: &[u8], salt: &[u8]) -> crate::Result<Vec<u8>> {
    let params = Params::new(14, Params::RECOMMENDED_R, Params::RECOMMENDED_P, 64)?;
    let mut password_hash = vec![0; 64];
    scrypt(password.as_ref(), &salt, &params, &mut password_hash)?;
    Ok(password_hash)
}

impl ActiveModel {
    pub fn from_unencrypted(user_id: u64, password: &str) -> crate::Result<Self> {
        let mut salt = vec![0; 64];
        OsRng.fill_bytes(&mut salt);
        let password_hash = Set(hash_password(password.as_ref(), &salt)?);
        Ok(Self {
            user_id: Set(user_id),
            salt: Set(salt),
            password_hash,
        })
    }
}

impl Entity {
    pub async fn verify_master_pass(
        user_id: u64,
        master_pass: &str,
        db: &DatabaseConnection,
    ) -> crate::Result<Option<bool>> {
        let model = match Self::find_by_id(user_id).one(db).await {
            Ok(Some(model)) => model,
            Ok(None) => return Ok(None),
            Err(err) => return Err(err.into()),
        };
        let password_hash = hash_password(master_pass.as_ref(), &model.salt)?;
        Ok(Some(password_hash == model.password_hash))
    }
}
