use crate::entity::{account, prelude::Account};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*, types::ParseMode};

use crate::handlers::{utils::package_handler, MainDialogue, State};

async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    name: String,
    master_pass: String,
) -> crate::Result<()> {
    dialogue.exit().await?;
    let user_id = msg.from().unwrap().id.0;
    let account = Account::find()
        .filter(account::Column::UserId.eq(user_id))
        .filter(account::Column::Name.eq(&name))
        .one(&db)
        .await?
        .unwrap();
    let (login, password) = account.decrypt(&master_pass)?;
    let message = format!("Name:\n`{name}`\nLogin:\n`{login}`\nPassword:\n`{password}`");
    bot.send_message(msg.chat.id, message)
        .parse_mode(ParseMode::MarkdownV2)
        .await?;
    Ok(())
}

async fn get_account_name(
    bot: Throttle<Bot>,
    msg: Message,
    _: DatabaseConnection,
    dialogue: MainDialogue,
    name: String,
) -> crate::Result<()> {
    let _ = bot.delete_message(msg.chat.id, msg.id).await;
    bot.send_message(msg.chat.id, "Send master password")
        .await?;
    dialogue
        .update(State::GetMasterPass(package_handler(
            move |bot, msg, db, dialogue, master_pass| {
                get_master_pass(bot, msg, db, dialogue, name, master_pass)
            },
        )))
        .await?;
    Ok(())
}

pub async fn get_account(
    bot: Throttle<Bot>,
    msg: Message,
    dialogue: MainDialogue,
) -> crate::Result<()> {
    bot.send_message(msg.chat.id, "Send account name").await?;
    dialogue
        .update(State::GetAccountName(package_handler(get_account_name)))
        .await?;
    Ok(())
}
